rm(list=ls())
graphics.off()

# required packages
require(lpSolve)
require(e1071)
require(rda)
require(gcda)

####################################################
### All the necessary changes shall be made here ###
# set working directory
setwd('~/GCDA/datasets/')
# Ridge.net or ARACNE ?
infernet <- "ridge.net" # or infernet <- "ARACNE"
# Which dataset ?
datachar <- "colon" # or datachar <- "lung" or datachar <- "prostate"
####################################################


# required functions
setwd("functions")
source("rda_short.R")
source("rda.cv2.R")
source("mysvm.R")
source("stdsvm.R")
source("nbsvm.R")
source("predict.stdsvm.R")
source("predict.nbsvm.R")
source("cv.stdsvm.R")
source("cv.nbsvm.R")
setwd("..")

dataset <- paste(datachar,"Cancer/",datachar,"Cancer.rda",sep="")
load(dataset) 

if (infernet == "ridge.net" ){
  gr1 <- g_R1
  gr2 <- g_R2
  gr <- g_R
} else if (infernet == "ARACNE") {
  gr1 <- g_A1 
  gr2 <- g_A2
  gr <- g_A
} else stop("Wrong inference method.")

n <- length(y) ; p <- ncol(x)

B <- 100
alphaopt <- scoresvm <- scorerda <- scstd <- scnb <- sclda <- scgcdalin <- scgcdaquad <- c()
alphas <- seq(0.1,0.9,l=5)
alphasopt <- matrix(NA,B,2)

for (i in 1:B){
  cat("Iteration nbr",i,"\b: ")
  train <- sample(n,2*n/3)
  xtrain <- x[train,] ; ytrain <- y[train]
  xtest <- x[-train,] ; ytest <- y[-train]
  train <- sample(1:n,3*n/4)
  cat("gcda quad, ")
  alphasopt[i,] <- cv.gcda(xtrain,ytrain,gr1=gr1,gr2=gr2,Alpha1=alphas,Alpha2=alphas,type="quadratic")$alphasopt
  resquad <- gcda(xtrain,ytrain,xtest,ytest,gr1=gr1,gr2=gr2,alpha1=alphasopt[i,1],alpha2=alphasopt[i,2],type="quadratic")
  scgcdaquad[i] <- resquad$error/length(ytest)
  cat("gcda lin, ")
  alphaopt[i] <- cv.gcda(xtrain,ytrain,gr=gr,Alpha=alphas,type="linear")$alphaopt
  reslin <- gcda(xtrain,ytrain, xtest,ytest,gr=gr,alpha=alphaopt[i],type="linear")
  scgcdalin[i] <- reslin$error/length(ytest)
  cat("svm, ")
  scoresvm[i] <-  sum(predict( mysvm(xtrain,ytrain),newdata=xtest)!=ytest)/length(ytest)
  cat("rda, ")
  scorerda[i] <-  rda_short(xtrain,ytrain,xtest,ytest)$error
  cat("STD-SVM, ")
  scstd[i]    <- predict.stdsvm(cv.stdsvm(xtrain,ytrain)$obj,xtest,ytest,scale=FALSE)$er
  cat("NB-SVM")
  scnb[i]     <- predict.nbsvm(cv.nbsvm(xtrain,ytrain,gr)$obj,xtest,ytest,scale=FALSE)$er
  cat("\n")
}
     
wdout <- paste(c("Data",datachar,gsub(":","_",as.character(Sys.time())),".pdf"),sep="",collapse="_")
dir.create(wdout)
setwd(wdout)

padj <- p.adjust(apply(x,2,function(vec) t.test(vec[y==-1],vec[y==1])$p.value))

write.table(rbind(c("p=",p),c("n=",n),c("pdiff=",sum(padj < 0.05)),c("B=",B)),file="params.txt",row.names=F,col.names=F)

res <-  rbind(c("Method","Mean","SD"),
c("score RDA : "     ,100 - 100*mean(scorerda)  ,sd(scorerda)),
c("score SVM : "     ,100 - 100*mean(scoresvm)  ,sd(scoresvm)),
c("score LP-SVM : "  ,100 - 100*mean(scstd)     ,sd(scstd)),
c("score NB-SVM : "  ,100 - 100*mean(scnb)      ,sd(scnb)),
c("score gcDA quad: ",100 - 100*mean(scgcdaquad),sd(scgcdaquad)),
c("score gcDA lin: " ,100 - 100*mean(scgcdalin) ,sd(scgcdalin)))

write.table(res,file="results.txt",row.names=F,col.names=F)

finaltab <- data.frame(SVM=100-100*(scoresvm),
                   RDA=100-100*(scorerda),
                   LPSVM=100-100*(scstd),
                   NBSVM=100-100*(scnb),
                   gcDAquad = 100-100*(scgcdaquad),
                   gcDAlin = 100-100*(scgcdalin))

pdf("boxplots.pdf")
boxplot(finaltab,notch=TRUE)
dev.off()

save(finaltab,file="finaltab.rda")
